@@ -6,7 +6,7 @@
  * Some ICCCM manager selection code derived from fvwm2,
  * Copyright (C) 2001 Dominik Vogt, Matthias Clasen, and fvwm2 team
  * Copyright (C) 2003 Rob Adams
- * Copyright (C) 2004, 2005 Elijah Newren
+ * Copyright (C) 2004-2006 Elijah Newren
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -187,6 +187,20 @@ reload_xinerama_infos (MetaScreen *screen)
 {
   MetaDisplay *display;
 
+  {
+    GList *tmp;
+
+    tmp = screen->workspaces;
+    while (tmp != NULL)
+      {
+        MetaWorkspace *space = tmp->data;
+
+        meta_workspace_invalidate_work_area (space);
+        
+        tmp = tmp->next;
+      }
+  }
+
   display = screen->display;
   
   if (screen->xinerama_infos)
@@ -348,20 +362,6 @@ reload_xinerama_infos (MetaScreen *screen)
 
   g_assert (screen->n_xinerama_infos > 0);
   g_assert (screen->xinerama_infos != NULL);
-
-  {
-    GList *tmp;
-
-    tmp = screen->workspaces;
-    while (tmp != NULL)
-      {
-        MetaWorkspace *space = tmp->data;
-
-        meta_workspace_invalidate_work_area (space);
-        
-        tmp = tmp->next;
-      }
-  }
 }
 
 MetaScreen*
@@ -534,13 +534,8 @@ meta_screen_new (MetaDisplay *display,
   screen->columns_of_workspaces = -1;
   screen->vertical_workspaces = FALSE;
   screen->starting_corner = META_SCREEN_TOPLEFT;
+  screen->compositor_data = NULL;
 
-  screen->compositor_windows = NULL;
-  screen->damage_region = None;
-  screen->root_picture = None;
-  screen->trans_pixmap = None;
-  screen->trans_picture = None;
-  
   {
     XFontStruct *font_info;
     XGCValues gc_values;
@@ -652,9 +647,6 @@ meta_screen_new (MetaDisplay *display,
       meta_workspace_activate (space, timestamp);
   }
 
-  meta_compositor_manage_screen (screen->display->compositor,
-                                 screen);
-  
   meta_verbose ("Added screen %d ('%s') root 0x%lx\n",
                 screen->number, screen->screen_name, screen->xroot);
   
@@ -673,8 +665,11 @@ meta_screen_free (MetaScreen *screen)
   
   meta_display_grab (display);
 
-  meta_compositor_unmanage_screen (screen->display->compositor,
-                                   screen);
+  if (screen->display->compositor)
+    {
+      meta_compositor_unmanage_screen (screen->display->compositor,
+				       screen);
+    }
   
   meta_display_unmanage_windows_for_screen (display, screen);
   
@@ -739,62 +734,104 @@ meta_screen_free (MetaScreen *screen)
   meta_display_ungrab (display);
 }
 
-void
-meta_screen_manage_all_windows (MetaScreen *screen)
+typedef struct
+{
+  Window		xwindow;
+  XWindowAttributes	attrs;
+} WindowInfo;
+
+static GList *
+list_windows (MetaScreen *screen)
 {
   Window ignored1, ignored2;
   Window *children;
-  int n_children;
-  int i;
+  guint n_children, i;
+  GList *result;
 
-  /* Must grab server to avoid obvious race condition */
-  meta_display_grab (screen->display);
-
-  meta_error_trap_push_with_return (screen->display);
-  
   XQueryTree (screen->display->xdisplay,
               screen->xroot,
               &ignored1, &ignored2, &children, &n_children);
 
-  if (meta_error_trap_pop_with_return (screen->display, TRUE) != Success)
+  result = NULL;
+  for (i = 0; i < n_children; ++i)
     {
-      meta_display_ungrab (screen->display);
-      return;
-    }
+      WindowInfo *info = g_new0 (WindowInfo, 1);
 
-  meta_stack_freeze (screen->stack);
-  i = 0;
-  while (i < n_children)
-    {
-      XWindowAttributes attrs;
-      
       meta_error_trap_push_with_return (screen->display);
       
       XGetWindowAttributes (screen->display->xdisplay,
-                            children[i], &attrs);
-      
-      if (meta_error_trap_pop_with_return (screen->display, TRUE) != Success)
-        {
+                            children[i], &info->attrs);
+
+      if (meta_error_trap_pop_with_return (screen->display, TRUE))
+	{
           meta_verbose ("Failed to get attributes for window 0x%lx\n",
                         children[i]);
+	  g_free (info);
         }
       else
         {
-          meta_window_new_with_attrs (screen->display, children[i], TRUE,
-                                      &attrs);
+	  info->xwindow = children[i];
+	}
 
-          meta_compositor_add_window (screen->display->compositor,
-                                      children[i], &attrs);
-        }
+      result = g_list_prepend (result, info);
+    }
 
-      ++i;
+  if (children)
+    XFree (children);
+
+  return g_list_reverse (result);
+}
+
+void
+meta_screen_manage_all_windows (MetaScreen *screen)
+{
+  GList *windows;
+  GList *list;
+
+  meta_display_grab (screen->display);
+  
+  windows = list_windows (screen);
+
+  meta_stack_freeze (screen->stack);
+  for (list = windows; list != NULL; list = list->next)
+    {
+      WindowInfo *info = list->data;
+
+      meta_window_new_with_attrs (screen->display, info->xwindow, TRUE,
+				  &info->attrs);
     }
   meta_stack_thaw (screen->stack);
 
+  g_list_foreach (windows, (GFunc)g_free, NULL);
+  g_list_free (windows);
+
   meta_display_ungrab (screen->display);
-  
-  if (children)
-    XFree (children);
+}
+
+void
+meta_screen_composite_all_windows (MetaScreen *screen)
+{
+  GList *windows, *list;
+
+  if (!screen->display->compositor)
+    return;
+
+  windows = list_windows (screen);
+
+  meta_stack_freeze (screen->stack);
+
+  for (list = windows; list != NULL; list = list->next)
+    {
+      WindowInfo *info = list->data;
+
+      meta_compositor_add_window (screen->display->compositor,
+				  info->xwindow, &info->attrs);
+    }
+
+  meta_stack_thaw (screen->stack);
+
+  g_list_foreach (windows, (GFunc)g_free, NULL);
+  g_list_free (windows);
 }
 
 MetaScreen*
@@ -990,7 +1027,7 @@ set_number_of_spaces_hint (MetaScreen *screen,
 
   data[0] = n_spaces;
 
-  meta_verbose ("Setting _NET_NUMBER_OF_DESKTOPS to %ld\n", data[0]);
+  meta_verbose ("Setting _NET_NUMBER_OF_DESKTOPS to %lu\n", data[0]);
 
   meta_error_trap_push (screen->display);
   XChangeProperty (screen->display->xdisplay, screen->xroot,
@@ -1011,7 +1048,7 @@ set_desktop_geometry_hint (MetaScreen *screen)
   data[0] = screen->rect.width;
   data[1] = screen->rect.height;
 
-  meta_verbose ("Setting _NET_DESKTOP_GEOMETRY to %ld, %ld\n", data[0], data[1]);
+  meta_verbose ("Setting _NET_DESKTOP_GEOMETRY to %lu, %lu\n", data[0], data[1]);
 
   meta_error_trap_push (screen->display);
   XChangeProperty (screen->display->xdisplay, screen->xroot,
@@ -1162,8 +1199,9 @@ meta_screen_update_cursor (MetaScreen *screen)
 }
 
 void
-meta_screen_ensure_tab_popup (MetaScreen *screen,
-                              MetaTabList type)
+meta_screen_ensure_tab_popup (MetaScreen      *screen,
+                              MetaTabList      list_type,
+                              MetaTabShowType  show_type)
 {
   MetaTabEntry *entries;
   GList *tab_list;
@@ -1175,7 +1213,7 @@ meta_screen_ensure_tab_popup (MetaScreen *screen,
     return;
 
   tab_list = meta_display_get_tab_list (screen->display,
-                                        type,
+                                        list_type,
                                         screen,
                                         screen->active_workspace);
   
@@ -1202,37 +1240,51 @@ meta_screen_ensure_tab_popup (MetaScreen *screen,
       entries[i].hidden = !meta_window_showing_on_its_workspace (window);
       entries[i].demands_attention = window->wm_state_demands_attention;
       
-      if (!window->minimized || !meta_window_get_icon_geometry (window, &r))
+      if (show_type == META_TAB_SHOW_INSTANTLY ||
+          !entries[i].hidden                   ||
+          !meta_window_get_icon_geometry (window, &r))
         meta_window_get_outer_rect (window, &r);
       
       entries[i].rect = r;
 
-      /* Find inside of highlight rectangle to be used
-       * when window is outlined for tabbing.
-       * This should be the size of the east/west frame,
-       * and the size of the south frame, on those sides.
-       * on the top it should be the size of the south frame
+      /* Find inside of highlight rectangle to be used when window is
+       * outlined for tabbing.  This should be the size of the
+       * east/west frame, and the size of the south frame, on those
+       * sides.  On the top it should be the size of the south frame
        * edge.
        */
-      if (window->frame)
-        {
-          int south = window->frame->rect.height - window->frame->child_y -
-            window->rect.height;
-          int east = window->frame->child_x;
-          entries[i].inner_rect.x = east;
-          entries[i].inner_rect.y = south;
-          entries[i].inner_rect.width = window->rect.width;
-          entries[i].inner_rect.height = window->frame->rect.height - south * 2;
-        }
-      else
-        {
-          /* Use an arbitrary border size */
 #define OUTLINE_WIDTH 5
-          entries[i].inner_rect.x = OUTLINE_WIDTH;
-          entries[i].inner_rect.y = OUTLINE_WIDTH;
-          entries[i].inner_rect.width = window->rect.width - OUTLINE_WIDTH * 2;
-          entries[i].inner_rect.height = window->rect.height - OUTLINE_WIDTH * 2;
-        }
+      /* Top side */
+      if (!entries[i].hidden &&
+          window->frame && window->frame->bottom_height > 0 &&
+          window->frame->child_y >= window->frame->bottom_height)
+        entries[i].inner_rect.y = window->frame->bottom_height;
+      else
+        entries[i].inner_rect.y = OUTLINE_WIDTH;
+
+      /* Bottom side */
+      if (!entries[i].hidden &&
+          window->frame && window->frame->bottom_height != 0)
+        entries[i].inner_rect.height = r.height
+          - entries[i].inner_rect.y - window->frame->bottom_height;
+      else
+        entries[i].inner_rect.height = r.height
+          - entries[i].inner_rect.y - OUTLINE_WIDTH;
+
+      /* Left side */
+      if (!entries[i].hidden && window->frame && window->frame->child_x != 0)
+        entries[i].inner_rect.x = window->frame->child_x;
+      else
+        entries[i].inner_rect.x = OUTLINE_WIDTH;
+
+      /* Right side */
+      if (!entries[i].hidden &&
+          window->frame && window->frame->right_width != 0)
+        entries[i].inner_rect.width = r.width
+          - entries[i].inner_rect.x - window->frame->right_width;
+      else
+        entries[i].inner_rect.width = r.width
+          - entries[i].inner_rect.x - OUTLINE_WIDTH;
       
       ++i;
       tmp = tmp->next;
@@ -1665,7 +1717,7 @@ meta_screen_update_workspace_layout (MetaScreen *screen)
       meta_XFree (list);
     }
 
-  meta_verbose ("Workspace layout rows = %d cols = %d orientation = %d starting corner = %d\n",
+  meta_verbose ("Workspace layout rows = %d cols = %d orientation = %d starting corner = %u\n",
                 screen->rows_of_workspaces,
                 screen->columns_of_workspaces,
                 screen->vertical_workspaces,
@@ -1708,7 +1760,7 @@ set_workspace_names (MetaScreen *screen)
                    screen->display->atom_net_desktop_names,
 		   screen->display->atom_utf8_string,
                    8, PropModeReplace,
-		   flattened->str, flattened->len);
+		   (unsigned char *)flattened->str, flattened->len);
   meta_error_trap_pop (screen->display, FALSE);
   
   g_string_free (flattened, TRUE);
